
#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

extern "C" {
  #include "php.h"
  #include "php_ini.h"
  #include "ext/standard/info.h"
  #include "php_fastbit.h"
  #include "zend_exceptions.h"
  #ifdef HAVE_FASTBIT_MYSQL
  #include "mysql.h"
  #endif
}

#include "iostream"
#include "ibis.h"
#include "selectClause.h"
#include "bord.h"
#include "tafel.h"

#define EXCEPTION_CATCHER(custom_message) catch(const std::exception& e) \
{\
  std::string message;\
  message+=custom_message;\
  message+=":";\
  message+=e.what();\
  char *sz;\
  sz = new char[message.length() + 1];\
  strcpy(sz, message.c_str());\
  zend_throw_exception(NULL,sz,0 TSRMLS_CC);\
  delete [] sz;\
}\
catch(char* s) \
{\
  std::string message;\
  message+=custom_message;\
  message+=": ";\
  message+=s;\
  char *sz;\
  sz = new char[message.length() + 1];\
  strcpy(sz, message.c_str());\
  zend_throw_exception(NULL,sz,0 TSRMLS_CC);\
  delete [] sz;\
}\
catch(...) \
{\
  std::string message;\
  message+=custom_message;\
  message+=":";\
  message+="Unhandled exception";\
  char *sz;\
  sz = new char[message.length() + 1];\
  strcpy(sz, message.c_str());\
  zend_throw_exception(NULL,sz,0 TSRMLS_CC);\
  delete [] sz;\
}

using namespace std;

/* If you declare any globals in php_fastbit.h uncomment this:
ZEND_DECLARE_MODULE_GLOBALS(fastbit)
*/

/* True global resources - no need for thread safety here */
static zend_object_handlers fastbit_handlers;
static zend_class_entry *fastbit_class;
static zend_class_entry *fastbit_partition_class;
static zend_class_entry *fastbit_table_class;

typedef struct _php_fastbit_partition_object
{
  zend_object std;
  ibis::part *part;
} php_fastbit_partition_object;

typedef struct _php_fastbit_table_object
{
  zend_object std;
  ibis::table *table;
  zval *partition;
} php_fastbit_table_object;

static void php_fastbit_partition_object_dtor(void *object, zend_object_handle handle TSRMLS_DC)
{
  php_fastbit_partition_object *intern = (php_fastbit_partition_object*)object;
  zend_object_std_dtor(&(intern->std) TSRMLS_CC);
  if (intern->part)
    delete intern->part;
  efree(intern);
}

static void php_fastbit_table_object_dtor(void *object, zend_object_handle handle TSRMLS_DC)
{
  php_fastbit_table_object *intern = (php_fastbit_table_object*)object;
  if (intern->partition)
    zend_objects_store_del_ref(intern->partition TSRMLS_CC);
  zend_object_std_dtor(&(intern->std) TSRMLS_CC);
  if (intern->table)
    delete intern->table;
  efree(intern);
}

static zend_object_value php_fastbit_partition_create_object(zend_class_entry *ce TSRMLS_DC)
{
  zend_object_value retval;
  php_fastbit_partition_object *intern;
  zval *tmp;

  intern=(php_fastbit_partition_object*)ecalloc(1,sizeof(php_fastbit_partition_object));
  zend_object_std_init(&(intern->std), ce TSRMLS_CC);
  zend_hash_copy(intern->std.properties,
      &ce->default_properties,
      (copy_ctor_func_t) zval_add_ref,
      (void *) &tmp, sizeof(zval *));

  retval.handle = zend_objects_store_put(intern,NULL,(zend_objects_free_object_storage_t)php_fastbit_partition_object_dtor,NULL TSRMLS_CC);
  retval.handlers = &fastbit_handlers;
  return retval;
}

static zend_object_value php_fastbit_table_create_object(zend_class_entry *ce TSRMLS_DC)
{
  zend_object_value retval;
  php_fastbit_table_object *intern;
  zval *tmp;

  intern=(php_fastbit_table_object*)ecalloc(1,sizeof(php_fastbit_table_object));
  zend_object_std_init(&(intern->std), ce TSRMLS_CC);
  zend_hash_copy(intern->std.properties,
      &ce->default_properties,
      (copy_ctor_func_t) zval_add_ref,
      (void *) &tmp, sizeof(zval *));

  retval.handle = zend_objects_store_put(intern,NULL,(zend_objects_free_object_storage_t)php_fastbit_table_object_dtor,NULL TSRMLS_CC);
  retval.handlers = &fastbit_handlers;
  return retval;
}

PHP_METHOD(fastbit,setverbosity)
{
  long verbosity;
  zval *thisp=getThis();

  if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC,"l",&verbosity)==FAILURE)
    return;
  
  ibis::gVerbose=verbosity;
}

#ifdef HAVE_FASTBIT_MYSQL
PHP_METHOD(fastbit,load)
{
  char *datadir,*hostname,*username,*password,*database,*select;
  int datadir_len,hostname_len,username_len,password_len,database_len,select_len;
  long port;
  
  Z_TYPE_P(return_value)=IS_BOOL;
  Z_LVAL_P(return_value)=0;

  if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC,"sslssss",&datadir,&datadir_len,&hostname,&hostname_len,&port,&username,&username_len,&password,&password_len,&database,&database_len,&select,&select_len)==FAILURE)
    return;

  MYSQL *conn;
  MYSQL_RES *res;
  MYSQL_ROW row;
  MYSQL_FIELD *field;

  conn = mysql_init(NULL);

  /* verbinden zur DB */
  if (!mysql_real_connect(conn, hostname, username, password, database, port, NULL, 0)) {
    php_error_docref(NULL TSRMLS_CC, E_WARNING, "Mysql Connect failed");
    return;
  }

  /* die SQL query senden*/
  if (mysql_query(conn, select)) {
    php_error_docref(NULL TSRMLS_CC, E_WARNING, "Mysql select failed: %s",mysql_error(conn));
    mysql_close(conn);
    return;
  }

  ibis::tablex* ta = ibis::tablex::create();
  ibis::table::row irow;

  res = mysql_store_result(conn);

  unsigned int i;
  unsigned long rowcount=0;
  if (mysql_num_rows(res)==0)
  {
    Z_LVAL_P(return_value)=0;
    goto xxx;
  }
  mysql_field_seek(res, 0);
  for(field = mysql_fetch_field(res), i = 0; field; field = mysql_fetch_field(res), i++)
  {
    switch(field->type)
    {
      case MYSQL_TYPE_LONGLONG:
      case MYSQL_TYPE_LONG:
        ta->addColumn(field->name,ibis::LONG);
        break;
      case MYSQL_TYPE_SHORT:
      case MYSQL_TYPE_TINY:
        ta->addColumn(field->name,ibis::SHORT);
        break;
      case MYSQL_TYPE_INT24:
        ta->addColumn(field->name,ibis::INT);
        break;
      case MYSQL_TYPE_FLOAT:
        ta->addColumn(field->name,ibis::FLOAT);
        break;
      case MYSQL_TYPE_DOUBLE:
      case MYSQL_TYPE_NEWDECIMAL:
        ta->addColumn(field->name,ibis::DOUBLE);
        break;
      case MYSQL_TYPE_STRING:
      case MYSQL_TYPE_VAR_STRING:
      case MYSQL_TYPE_BLOB:
        ta->addColumn(field->name,ibis::TEXT);
        break;
/*      case MYSQL_TYPE_DATETIME:
        ta->addColumn(field->name,ibis::LONG);
        break;
*/
      default:
        php_error_docref(NULL TSRMLS_CC, E_WARNING, "Unsupported MySQL type: %d in field %s",(int)field->type,field->name);
        goto xxx;
    }
  }
  
  while ((row = mysql_fetch_row(res)))
  {
    irow.clear();
    mysql_field_seek(res, 0);
	for(field = mysql_fetch_field(res), i = 0; field; field = mysql_fetch_field(res), i++)
    {
      if (row[i]==NULL)
      {
        php_error_docref(NULL TSRMLS_CC, E_WARNING, "Field %s row %d is NULL which is unsupported",field->name,rowcount);
        goto xxx;
      }
        
      //it=cols.find(field->name);
      switch(field->type)
      {
        case MYSQL_TYPE_LONGLONG:
        case MYSQL_TYPE_LONG:
          irow.longsnames.push_back(field->name);
/*          if (row[i]==NULL)
            irow.longsvalues.push_back(0x7FFFFFFFFFFFFFFFLL);
          else
*/
          irow.longsvalues.push_back(strtol(row[i],NULL,10));
          break;
        case MYSQL_TYPE_SHORT:
        case MYSQL_TYPE_TINY:
          irow.shortsnames.push_back(field->name);
/*
          if (row[i]==NULL)
            irow.shortsvalues.push_back((int16_t)0x7FFF);
//            irow.shortsvalues.push_back((int16_t)0);
          else
*/
          irow.shortsvalues.push_back((int16_t)strtol(row[i],NULL,10));
          break;
        case MYSQL_TYPE_INT24:
          irow.intsnames.push_back(field->name);
/*
          if (row[i]==NULL)
            irow.intsvalues.push_back((int32_t)0x7FFFFFFF);
//            irow.intsvalues.push_back((int32_t)0);
          else
*/
          irow.intsvalues.push_back((int32_t)strtol(row[i],NULL,10));
          break;
        case MYSQL_TYPE_FLOAT:
          irow.floatsnames.push_back(field->name);
/*
          if (row[i]==NULL)
            irow.floatsvalues.push_back(std::numeric_limits<float>::quiet_NaN());
            //irow.floatsvalues.push_back((float)0);
          else
*/
          irow.floatsvalues.push_back((float)strtof(row[i],NULL));
          break;
        case MYSQL_TYPE_DOUBLE:
        case MYSQL_TYPE_NEWDECIMAL:
          irow.doublesnames.push_back(field->name);
/*
          if (row[i]==NULL)
            irow.doublesvalues.push_back(std::numeric_limits<double>::quiet_NaN());
//            irow.doublesvalues.push_back((double)0);
          else
*/
          irow.doublesvalues.push_back((double)strtod(row[i],NULL));
          break;
        case MYSQL_TYPE_STRING:
        case MYSQL_TYPE_VAR_STRING:
        case MYSQL_TYPE_BLOB:
          irow.textsnames.push_back(field->name);
/*
          if (row[i]==NULL)
            irow.textsvalues.push_back("");
          else
*/
          irow.textsvalues.push_back(row[i]);
          break; 
/*        case MYSQL_TYPE_DATETIME:
          irow.longsnames.push_back(field->name);
          irow.longsvalues.push_back(0);
          break;
*/
        default:
          php_error_docref(NULL TSRMLS_CC, E_WARNING, "Unsupported MySQL type (row): %d",field->type);
          goto xxx;
      }
    }
    ta->appendRow(irow);
    rowcount++;
  }
  /* Speicher freigeben und Verbindung trennen */
  ta->write(datadir);
  Z_TYPE_P(return_value)=IS_BOOL;
  Z_LVAL_P(return_value)=1;
  xxx:
    mysql_free_result(res);
    mysql_close(conn);
    if (ta)
      delete ta;
}
#endif

PHP_METHOD(fastbit_partition,__construct)
{
  char *datadir;
  int datadir_len;
  zval *thisp=getThis();

  if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC,"s",&datadir,&datadir_len)==FAILURE)
    return;
  
  php_fastbit_partition_object *obj=(php_fastbit_partition_object*)zend_object_store_get_object(thisp TSRMLS_CC);
  try
  {
    obj->part=new ibis::part(datadir,true);
  }
  EXCEPTION_CATCHER("Couldn't create partition");
  if (!obj->part)
    zend_throw_exception(NULL,"Creating partition failed",0 TSRMLS_CC);
}

PHP_METHOD(fastbit_table,__construct)
{
  zval *thisp=getThis();
  zval *partition_zval;

  if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC,"o",&partition_zval)==FAILURE)
    return;
  
  php_fastbit_partition_object *partition=(php_fastbit_partition_object*)zend_object_store_get_object(partition_zval TSRMLS_CC);

  php_fastbit_table_object *obj=(php_fastbit_table_object*)zend_object_store_get_object(thisp TSRMLS_CC);
  obj->partition=0;
  try
  {
    obj->table=ibis::table::create(*partition->part);
    obj->partition=partition_zval;
    zval_add_ref(&partition_zval);
  }
  EXCEPTION_CATCHER("Couldn't create table");
  if (!obj->table)
    zend_throw_exception(NULL,"Creating table failed",0 TSRMLS_CC);
}


PHP_METHOD(fastbit_table,select)
{
  char *selects,*wheres;
  int select_len,where_len;
  zval *thisp=getThis();
  zval *select_zval;
  php_fastbit_table_object *select_object;

  if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC,"ss",&selects,&select_len,&wheres,&where_len)==FAILURE)
    return;

  php_fastbit_table_object *intern = (php_fastbit_table_object*)zend_object_store_get_object(thisp TSRMLS_CC);
  ibis::table* table=intern->table;
  ibis::table* select=0;

  try
  {
    select=table->select(selects,wheres);
  }
  catch (...)
  {
    zend_throw_exception(NULL,"Select failed",0 TSRMLS_CC);
    return;
  }
  if (!select)
  {
    zend_throw_exception(NULL,"Select failed",0 TSRMLS_CC);
    return;
  }
  
  Z_TYPE_P(return_value)=IS_OBJECT;
  object_init_ex(return_value,fastbit_table_class);
  select_object=(php_fastbit_table_object*)zend_object_store_get_object(return_value TSRMLS_CC);
  select_object->table=select;
}

PHP_METHOD(fastbit_table,groupby)
{
  char *groupbys;
  int groupby_len;
  php_fastbit_table_object *groupby_object;

  zval *thisp=getThis();
  if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC,"s",&groupbys,&groupby_len)==FAILURE)
    return;

  php_fastbit_table_object *intern = (php_fastbit_table_object*)zend_object_store_get_object(thisp TSRMLS_CC);
  ibis::table* table=intern->table;
  ibis::table* groupby=0;
  try
  {
    ibis::selectClause sel(groupbys);
    groupby=((ibis::bord*)table)->groupby(sel);
  }
  catch (...)
  {
    zend_throw_exception(NULL,"Groupby failed",0 TSRMLS_CC);
    return;
  }
  if (!groupby)
  {
    zend_throw_exception(NULL,"Groupby failed",0 TSRMLS_CC);
    return;
  }
  Z_TYPE_P(return_value)=IS_OBJECT;
  object_init_ex(return_value,fastbit_table_class);
  groupby_object=(php_fastbit_table_object*)zend_object_store_get_object(return_value TSRMLS_CC);
  groupby_object->table=groupby;
}

PHP_METHOD(fastbit_table,orderby)
{
  char *orderbys;
  int orderby_len;

  zval *thisp=getThis();
  if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC,"s",&orderbys,&orderby_len)==FAILURE)
    return;

  php_fastbit_table_object *intern = (php_fastbit_table_object*)zend_object_store_get_object(thisp TSRMLS_CC);
  ibis::table* table=intern->table;
  try
  {
    table->orderby(orderbys);
  }
  catch (...)
  {
    zend_throw_exception(NULL,"Orderby failed",0 TSRMLS_CC);
    return;
  }
}

PHP_METHOD(fastbit_table,nrows)
{
  zval *thisp=getThis();
  php_fastbit_table_object *intern = (php_fastbit_table_object*)zend_object_store_get_object(thisp TSRMLS_CC);
  RETURN_LONG(intern->table->nRows());
}

PHP_METHOD(fastbit_table,ncolumns)
{
  zval *thisp=getThis();
  php_fastbit_table_object *intern = (php_fastbit_table_object*)zend_object_store_get_object(thisp TSRMLS_CC);
  RETURN_LONG(intern->table->nColumns());
}

PHP_METHOD(fastbit_table,columnnames)
{
  zval *thisp=getThis();
  php_fastbit_table_object *intern = (php_fastbit_table_object*)zend_object_store_get_object(thisp TSRMLS_CC);
  
  array_init(return_value);
  ibis::table::stringList nms = intern->table->columnNames();
#if FASTBIT_IBIS_INT_VERSION >= 1010900
  ibis::array_t<const char*>::iterator it;
#else
  vector<const char*>::iterator it;
#endif
  it=nms.begin();
  while (it!=nms.end())
  {
    add_next_index_string(return_value, (char*)*it, 1);
    ++it;
  }
}

PHP_METHOD(fastbit_table,columntypes)
{
  zval *thisp=getThis();
  php_fastbit_table_object *intern = (php_fastbit_table_object*)zend_object_store_get_object(thisp TSRMLS_CC);
  
  array_init(return_value);
  ibis::table::typeList tps = intern->table->columnTypes();
#if FASTBIT_IBIS_INT_VERSION >= 1010900
  ibis::array_t<ibis::TYPE_T>::iterator it;
#else
  vector<ibis::TYPE_T>::iterator it;
#endif
  it=tps.begin();
  while (it!=tps.end())
  {
    add_next_index_long(return_value, *it);
    ++it;
  }
}

PHP_METHOD(fastbit_table,gettableresult)
{
  zval *thisp=getThis();
  php_fastbit_table_object *intern = (php_fastbit_table_object*)zend_object_store_get_object(thisp TSRMLS_CC);

  ibis::table *result=intern->table;
    array_init(return_value);
    ibis::table::cursor *cur;
    cur=result->createCursor();
    zval *value;
    zval *row;

    ibis::table::stringList nms = result->columnNames();
    ibis::table::typeList tps = result->columnTypes();

    while (0 == cur->fetch()) {
      ALLOC_INIT_ZVAL(row);
      array_init(row);
      for (uint32_t j=0;j<cur->nColumns();++j)
      {
        ALLOC_INIT_ZVAL(value);
        if (tps[j]==ibis::INT)
        {
          int32_t ivalue;
          cur->getColumnAsInt(nms[j],ivalue);
          Z_TYPE_P(value)=IS_LONG;
          Z_LVAL_P(value)=(long)ivalue;
        }
        else if (tps[j]==ibis::UINT)
        {
          uint32_t ivalue;
          cur->getColumnAsUInt(nms[j],ivalue);
          Z_TYPE_P(value)=IS_DOUBLE;
          Z_DVAL_P(value)=(double)ivalue;
        }
        else if (tps[j]==ibis::LONG)
        {
          int64_t ivalue;
          cur->getColumnAsLong(nms[j],ivalue);
          Z_TYPE_P(value)=IS_DOUBLE;
          Z_DVAL_P(value)=(double)ivalue;
        }
        else if (tps[j]==ibis::ULONG)
        {
          uint64_t ivalue;
          cur->getColumnAsULong(nms[j],ivalue);
          Z_TYPE_P(value)=IS_DOUBLE;
          Z_DVAL_P(value)=(double)ivalue;
        }
        else if (tps[j]==ibis::SHORT)
        {
          int16_t ivalue;
          cur->getColumnAsShort(nms[j],ivalue);
          Z_TYPE_P(value)=IS_LONG;
          Z_LVAL_P(value)=(long)ivalue;
        }
        else if (tps[j]==ibis::USHORT)
        {
          uint16_t ivalue;
          cur->getColumnAsUShort(nms[j],ivalue);
          Z_TYPE_P(value)=IS_LONG;
          Z_LVAL_P(value)=(long)ivalue;
        }
        else if (tps[j]==ibis::DOUBLE)
        {
          double ivalue;
          cur->getColumnAsDouble(nms[j],ivalue);
          Z_TYPE_P(value)=IS_DOUBLE;
          Z_DVAL_P(value)=(double)ivalue;
        }
        else if (tps[j]==ibis::FLOAT)
        {
          float ivalue;
          cur->getColumnAsFloat(nms[j],ivalue);
          Z_TYPE_P(value)=IS_DOUBLE;
          Z_DVAL_P(value)=(double)ivalue;
        }
        else if (tps[j]==ibis::TEXT)
        {
          std::string ivalue;
          char* text;
          cur->getColumnAsString(nms[j],ivalue);
          Z_TYPE_P(value)=IS_STRING;
          Z_STRLEN_P(value)=ivalue.length();
          Z_STRVAL_P(value)=estrdup(ivalue.c_str());
        }
        else
        {
          Z_TYPE_P(value)=IS_NULL;
          php_error_docref(NULL TSRMLS_CC, E_WARNING, "Unsupported type %d (%s)",tps[j],ibis::TYPESTRING[tps[j]]);
        }

        add_assoc_zval(row,(char*)nms[j],value);
      }
      add_next_index_zval(return_value, row);
    }
}

ZEND_BEGIN_ARG_INFO_EX(arginfo_fastbit_partition___construct, 0, 0, 1)
  ZEND_ARG_INFO(0, datadir)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_fastbit_table___construct, 0, 0, 1)
  ZEND_ARG_INFO(0, partition)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_fastbit_table_select, 0, 0, 2)
  ZEND_ARG_INFO(0, select)
  ZEND_ARG_INFO(0, where)
ZEND_END_ARG_INFO()

#ifdef HAVE_FASTBIT_MYSQL
ZEND_BEGIN_ARG_INFO_EX(arginfo_fastbit_load, 0, 0, 7)
  ZEND_ARG_INFO(0,datadir)
  ZEND_ARG_INFO(0,hostname)
  ZEND_ARG_INFO(0,port)
  ZEND_ARG_INFO(0,username)
  ZEND_ARG_INFO(0,password)
  ZEND_ARG_INFO(0,database)
  ZEND_ARG_INFO(0,select)
ZEND_END_ARG_INFO()
#endif

/* {{{ fastbit_class_functions[] */
zend_function_entry fastbit_class_functions[] = {
  PHP_ME(fastbit, setverbosity,             NULL,                                   ZEND_ACC_PUBLIC | ZEND_ACC_STATIC)
  #ifdef HAVE_FASTBIT_MYSQL
  PHP_ME(fastbit, load,                     NULL,                                   ZEND_ACC_PUBLIC | ZEND_ACC_STATIC)
  #endif
  {NULL, NULL, NULL}
};
/* }}} */

/* {{{ fastbit_partition_class_functions[] */
zend_function_entry fastbit_partition_class_functions[] = {
  PHP_ME(fastbit_partition, __construct,              NULL,                                   ZEND_ACC_PUBLIC)
  {NULL, NULL, NULL}
};
/* }}} */

/* {{{ fastbit_table_class_functions[] */
zend_function_entry fastbit_table_class_functions[] = {
  PHP_ME(fastbit_table, __construct,              NULL,                                   ZEND_ACC_PUBLIC)
  PHP_ME(fastbit_table, select,                   NULL,                                   ZEND_ACC_PUBLIC)
  PHP_ME(fastbit_table, groupby,                  NULL,                                   ZEND_ACC_PUBLIC)
  PHP_ME(fastbit_table, orderby,                  NULL,                                   ZEND_ACC_PUBLIC)
  PHP_ME(fastbit_table, nrows,                    NULL,                                   ZEND_ACC_PUBLIC)
  PHP_ME(fastbit_table, ncolumns,                 NULL,                                   ZEND_ACC_PUBLIC)
  PHP_ME(fastbit_table, columnnames,              NULL,                                   ZEND_ACC_PUBLIC)
  PHP_ME(fastbit_table, columntypes,              NULL,                                   ZEND_ACC_PUBLIC)
  PHP_ME(fastbit_table, gettableresult,           NULL,                                   ZEND_ACC_PUBLIC)
  {NULL, NULL, NULL}
};
/* }}} */

/* {{{ fastbit_module_entry
 */
zend_module_entry fastbit_module_entry = {
#if ZEND_MODULE_API_NO >= 20010901
	STANDARD_MODULE_HEADER,
#endif
	"fastbit",
	NULL,
	PHP_MINIT(fastbit),
	PHP_MSHUTDOWN(fastbit),
	PHP_RINIT(fastbit),		/* Replace with NULL if there's nothing to do at request start */
	PHP_RSHUTDOWN(fastbit),	/* Replace with NULL if there's nothing to do at request end */
	PHP_MINFO(fastbit),
#if ZEND_MODULE_API_NO >= 20010901
	FASTBIT4PHP_VERSION, /* Replace with version number for your extension */
#endif
	STANDARD_MODULE_PROPERTIES
};
/* }}} */

#ifdef COMPILE_DL_FASTBIT
ZEND_GET_MODULE(fastbit)
#endif

/* {{{ PHP_INI
 */
/* Remove comments and fill if you need to have entries in php.ini
PHP_INI_BEGIN()
    STD_PHP_INI_ENTRY("fastbit.global_value",      "42", PHP_INI_ALL, OnUpdateLong, global_value, zend_fastbit_globals, fastbit_globals)
    STD_PHP_INI_ENTRY("fastbit.global_string", "foobar", PHP_INI_ALL, OnUpdateString, global_string, zend_fastbit_globals, fastbit_globals)
PHP_INI_END()
*/
/* }}} */

/* {{{ php_fastbit_init_globals
 */
/* Uncomment this function if you have INI entries
static void php_fastbit_init_globals(zend_fastbit_globals *fastbit_globals)
{
	fastbit_globals->global_value = 0;
	fastbit_globals->global_string = NULL;
}
*/
/* }}} */

void _register_fastbit_class(TSRMLS_D)
{
  zend_class_entry fastbit_ce;
  INIT_CLASS_ENTRY(fastbit_ce, "fastbit", fastbit_class_functions);
  fastbit_class = zend_register_internal_class(&fastbit_ce TSRMLS_CC);
  memcpy(&fastbit_handlers, zend_get_std_object_handlers(),sizeof(zend_object_handlers));
  fastbit_class->ce_flags |= ZEND_ACC_FINAL;
}

void _register_fastbit_partition_class(TSRMLS_D)
{
  zend_class_entry fastbit_partition_ce;
  INIT_CLASS_ENTRY(fastbit_partition_ce, "fastbit_partition", fastbit_partition_class_functions);
  fastbit_partition_class = zend_register_internal_class(&fastbit_partition_ce TSRMLS_CC);
  fastbit_partition_class->create_object = php_fastbit_partition_create_object;
  memcpy(&fastbit_handlers, zend_get_std_object_handlers(),sizeof(zend_object_handlers));
  fastbit_partition_class->ce_flags |= ZEND_ACC_FINAL;
}

void _register_fastbit_table_class(TSRMLS_D)
{
  zend_class_entry fastbit_table_ce;
  INIT_CLASS_ENTRY(fastbit_table_ce, "fastbit_table", fastbit_table_class_functions);
  fastbit_table_class = zend_register_internal_class(&fastbit_table_ce TSRMLS_CC);
  fastbit_table_class->create_object = php_fastbit_table_create_object;
  memcpy(&fastbit_handlers, zend_get_std_object_handlers(),sizeof(zend_object_handlers));
  fastbit_table_class->ce_flags |= ZEND_ACC_FINAL;
}

/* {{{ PHP_MINIT_FUNCTION
 */
PHP_MINIT_FUNCTION(fastbit)
{
  /* If you have INI entries, uncomment these lines 
     REGISTER_INI_ENTRIES();
   */
  _register_fastbit_class(TSRMLS_C);
  _register_fastbit_partition_class(TSRMLS_C);
  _register_fastbit_table_class(TSRMLS_C);

  REGISTER_LONG_CONSTANT("FASTBIT_TYPE_UNKNOWN_TYPE",ibis::UNKNOWN_TYPE,CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("FASTBIT_TYPE_OID",ibis::OID,CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("FASTBIT_TYPE_BYTE",ibis::BYTE,CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("FASTBIT_TYPE_UBYTE",ibis::UBYTE,CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("FASTBIT_TYPE_SHORT",ibis::SHORT,CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("FASTBIT_TYPE_USHORT",ibis::USHORT,CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("FASTBIT_TYPE_INT",ibis::INT,CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("FASTBIT_TYPE_UINT",ibis::UINT,CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("FASTBIT_TYPE_LONG",ibis::LONG,CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("FASTBIT_TYPE_ULONG",ibis::ULONG,CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("FASTBIT_TYPE_FLOAT",ibis::FLOAT,CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("FASTBIT_TYPE_DOUBLE",ibis::DOUBLE,CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("FASTBIT_TYPE_CATEGORY",ibis::CATEGORY,CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("FASTBIT_TYPE_TEXT",ibis::TEXT,CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("FASTBIT_TYPE_BLOB",ibis::BLOB,CONST_CS | CONST_PERSISTENT);
  return SUCCESS;
}

/* }}} */

/* {{{ PHP_MSHUTDOWN_FUNCTION
 */
PHP_MSHUTDOWN_FUNCTION(fastbit)
{
	/* uncomment this line if you have INI entries
	UNREGISTER_INI_ENTRIES();
	*/
	return SUCCESS;
}
/* }}} */

/* Remove if there's nothing to do at request start */
/* {{{ PHP_RINIT_FUNCTION
 */
PHP_RINIT_FUNCTION(fastbit)
{
	return SUCCESS;
}
/* }}} */

/* Remove if there's nothing to do at request end */
/* {{{ PHP_RSHUTDOWN_FUNCTION
 */
PHP_RSHUTDOWN_FUNCTION(fastbit)
{
	return SUCCESS;
}
/* }}} */

/* {{{ PHP_MINFO_FUNCTION
 */
PHP_MINFO_FUNCTION(fastbit)
{
	php_info_print_table_start();
	php_info_print_table_row(2, "fastbit support", "enabled");
	php_info_print_table_row(2, "fastbit4php version", FASTBIT4PHP_VERSION);
	php_info_print_table_row(2, "fastbit version", PACKAGE_VERSION);
	php_info_print_table_end();

	/* Remove comments if you have entries in php.ini
	DISPLAY_INI_ENTRIES();
	*/
}
/* }}} */


/* The previous line is meant for vim and emacs, so it can correctly fold and 
   unfold functions in source code. See the corresponding marks just before 
   function definition, where the functions purpose is also documented. Please 
   follow this convention for the convenience of others editing your code.
*/


/*
 * Local variables:
 * tab-width: 4
 * c-basic-offset: 4
 * End:
 * vim600: noet sw=4 ts=4 fdm=marker
 * vim<600: noet sw=4 ts=4
 */
